/* cube.cpp
 *
 * Pieter Eendebak ( pte@ddsw.nl )
 *
 */

#include <qpntarry.h>
#include <stdlib.h>
#include <math.h>

#include <qfiledlg.h>
#include <qdstream.h>
#include <qtstream.h>

#include <kfm.h>

#include "includes.h"
#include "defines.h"
#include "cube.moc"

Cube::Cube ( QWidget *parent, const char *name )
		: QWidget (parent, name)
{
	const int minimum=80;
	setMinimumSize(minimum,minimum);

	map = new QPixmap(minimum,minimum);
	pt = new QPainter();

	initData();

	// start random number generator
	#ifdef linux
		srandom( time->msecsTo( QTime::currentTime() ) );
	#elif sparc
		srand( time->msecsTo( QTime::currentTime() ) );
	#else
		debug("Unsupported achitecture, please mail pte@ddsw.nl") ;
		srand( time->msecsTo( QTime::currentTime() ) );
	#endif
}
	
Cube::~Cube()
{
	// delete cube data
	delete bgcolor;
	delete edge;
	delete [] colorList;
	delete [] sideVec;
	delete [] corners;

	// delete misc
	delete time;

	// delete widgets
	delete map;
	delete pt;
}

const int Cube::SHADE = 0 ;
const int Cube::NO_SHADE = 0 ;

const int Cube::sides[24]={4,5,6,7,  3,2,1,0,  0,1,5,4,   1,2,6,5,
			2,3,7,6,  0,4,7,3 };
const int Cube::nextSide[24]={2,3,4,5,  4,3,2,5,  1,3,0,5,  1,4,0,2,
				1,5,0,3,  2,0,4,1};
const int Cube::mainBlocks[24] = {0,3,0,3,  0,3,0,3,  0,3,0,3,  0,3,0,3,
				 0,3,0,3,  0,3,0,3 };

const int Cube::twistDir[24] = {-1,1,-1,1, -1,1,-1,1, 1,1,1,1, 1,-1,1,-1,
				1,1,1,1, -1,1,-1,1};
const int Cube::colDir[6]={-1, -1, 1, -1, 1, -1};
const int Cube::circleOrder[8]={0,1,2,5,8,7,6,3};

void Cube::clean()
{
	phibase=0;
	twisting=false;
	naturalstate=true;
	for(int tmp=0;tmp<54;tmp++) sideCols[tmp]=(int)(tmp/9);
	
	repaint(false);
	
	moves=0;
	timestarted=false;

	emit stateChange();
}

void Cube::newCube()
{
	eye.set(0.3651,0.1826,-0.9129);
	eX.set(0.9309,-0.0716,0.3581);
	eY.vectorProd(eye, eX);
	eY.Normalize();

	clean();
}

void Cube::setColor(QColor color, int slot, bool redraw)
{
	// some checking
	if ( slot > 5 ) return;

	switch(slot)
	{
		case BACKGROUND: bgcolor->setRgb( color.red(),
					color.green(),
					color.blue() );
					break;
		case EDGE:	edge->setRgb( color.red(),
					color.green(),
					color.blue() );
				break;

		// here we set the colors
		default :	int red = color.red();
				int blue = color.blue();
				int green = color.green();

				createColor( slot, red, green, blue ) ; 
 				break;
	}

	if (redraw) repaint(false);
}

void Cube::createColor( int slot, int r, int g, int b )
{
	int level = getShading() ;
	if ( level == Cube::NO_SHADE )
		level = 0 ;
	int border = (level*20)/2 ; 
	if (r<border) r=border;
	if (g<border) g=border;
	if (b<border) b=border;
				
	for( int tmp=0;tmp<20;tmp++)
	{
		colorList[(20*slot)+tmp]->setRgb(
			r-((level*(19-tmp))/2),
			g-((level*(19-tmp))/2),
			b-((level*(19-tmp))/2) ) ;
	}
}


void Cube::redoColors()
{
	for( int i=0; i<6; i++ )
	{
		int r = colorList[(20*i)+19]->red() ;
		int g = colorList[(20*i)+19]->green() ;
		int b = colorList[(20*i)+19]->blue();
		createColor( i, r, g, b ) ;
	}
}

void Cube::setShading( int level)
{
	if ( level>=0 && level<=10 ) 
	{
		shading=level ;
		redoColors() ;
	}
	else	shading = Cube::NO_SHADE ;
}

int Cube::getShading()
{
	return shading;
}

void Cube::resetColors( bool redraw )
{
	setColor( backgroundColor(), BACKGROUND, false);
	setColor( QColor(black), EDGE, false);

	setColor( QColor(red), 0, false );
	setColor( QColor(white), 1, false );
	setColor( QColor(green), 2, false );
	setColor( QColor(yellow), 3, false );
	setColor( QColor(blue), 4, false );
	setColor( QColor(magenta), 5, false );

	if( redraw ) repaint(false);
}

const QColor Cube::getColorList( int nr )
{
	if ( nr>=0 && nr <120 )
		return *colorList[nr];
	else return QColor();
}

const QColor Cube::getColor( int i )
{
	if ( i>=0 && i<=5 )
		return *colorList[i*20+19];
	else return QColor();
}

const QColor Cube::getBackgroundColor()
{
	return QColor( *bgcolor );
}

const QColor Cube::getEdgeColor()
{
	return QColor ( *edge );
}


double Cube::getRandom()
{
	#ifdef linux
		return ( (double)(random()) / (RAND_MAX+1.0) ) ;
	#elif solaris
		return ( ((double)rand()) / (RAND_MAX+1.0) ) ;
	#else
		return ( ((double)rand()) / (RAND_MAX+1.0) ) ;
	#endif
}

void Cube::scramble()
{

	twisting=false;
	naturalstate=true;
	for(int tmp=0;tmp<25;tmp++)
		colorTwist( (int)( 6*getRandom() ),
				1+(int)(3* ( getRandom() ) ),
				false );

	moves=0;
	timestarted=false;

	emit stateChange();
	repaint( false );
}


void Cube::paintEvent ( QPaintEvent * )
{
	map->fill( *bgcolor );
	
	dragReg=0;

	if (naturalstate)
	{
		fixBlock(eye, eX, eY, corners, mainBlocks, 0);
	}
	else
	{
		Teye=eye;
		TeX=eX;
		Cphi= cos(phi+phibase);
		Sphi= -sin(phi+phibase);
		switch(twistSide)	// twist around which axis?
		{
		   case 0:	// z
				Teye.x()=Cphi*eye.x()+Sphi*eye.y();
				TeX.x()=Cphi*eX.x()+Sphi*eX.y();
				Teye.y()=-Sphi*eye.x()+Cphi*eye.y();
				TeX.y()=-Sphi*eX.x()+Cphi*eX.y();
				break;
		   case 1:	// -z
				Teye.x()=Cphi*eye.x()-Sphi*eye.y();
				TeX.x()=Cphi*eX.x()-Sphi*eX.y();
				Teye.y()=Sphi*eye.x()+Cphi*eye.y();
				TeX.y()=Sphi*eX.x()+Cphi*eX.y();
				break;
		   case 2:	// -y
				Teye.x()=Cphi*eye.x()-Sphi*eye.z();
				TeX.x()=Cphi*eX.x()-Sphi*eX.z();
				Teye.z()=Sphi*eye.x()+Cphi*eye.z();
				TeX.z()=Sphi*eX.x()+Cphi*eX.z();
				break;
		   case 3:	// x
				Teye.y()=Cphi*eye.y()+Sphi*eye.z();
				TeX.y()=Cphi*eX.y()+Sphi*eX.z();
				Teye.z()=-Sphi*eye.y()+Cphi*eye.z();
				TeX.z()=-Sphi*eX.y()+Cphi*eX.z();
				break;
		   case 4:	// y
				Teye.x()=Cphi*eye.x()+Sphi*eye.z();
				TeX.x()=Cphi*eX.x()+Sphi*eX.z();
				Teye.z()=-Sphi*eye.x()+Cphi*eye.z();
				TeX.z()=-Sphi*eX.x()+Cphi*eX.z();
				break;
		   case 5:	// -x
				Teye.y()=Cphi*eye.y()-Sphi*eye.z();
				TeX.y()=Cphi*eX.y()-Sphi*eX.z();
				Teye.z()=Sphi*eye.y()+Cphi*eye.z();
				TeX.z()=Sphi*eX.y()+Cphi*eX.z();
				break;
		   default:	break;

		}

		TeY.vectorProd(Teye,TeX);
		// Top facing away?
		if ( eye.scalProd( *sideVec[twistSide] )<0)
		{
			fixBlock( Teye, TeX, TeY, topCorners,
					topBlocks, 2);
			fixBlock( eye, eX, eY, botCorners,
					botBlocks, 1);
		}
		else
		{
			fixBlock( eye, eX, eY, botCorners,
					botBlocks, 1);
			fixBlock( Teye, TeX, TeY, topCorners,
					topBlocks, 2);
		}

	}

	bitBlt( this, 0, 0, map);	// show the new cube
}	
	
void Cube::resizeEvent ( QResizeEvent * )
{
	resizeEvent();
}

void Cube::resizeEvent ()
{
	QSize s = size();
	
	hoog = s.height();
	breed = s.width();
	grootte = ( (hoog<breed) ? hoog : breed );

	map->resize( breed, hoog );

	repaint();
}

void Cube::initData()
{
	int tmp;

	// start the timer
	time = new QTime();
	timestarted=false;
	
	rect.resize(4);

	// set the colors
	shading= Cube::NO_SHADE ;
	bgcolor = new QColor();
	edge = new QColor();

	colorList = new QColor *[120];
	for(tmp=0;tmp<120;tmp++) colorList[tmp] = new QColor();
	resetColors();

	// set up some data structures...
	sideVec = new Vector *[6];
	for(tmp=0;tmp<6;tmp++) sideVec[tmp]=new Vector();
	sideVec[0]->set(0,0,1);
	sideVec[1]->set(0,0,-1);
	sideVec[2]->set(0,-1,0);
	sideVec[3]->set(1,0,0);
	sideVec[4]->set(0,1,0);
	sideVec[5]->set(-1,0,0);

	corners = new Vector *[8];
	for(tmp=0;tmp<8;tmp++) corners[tmp]=new Vector();
	corners[0]->set(-1,-1,-1);
	corners[1]->set( 1,-1,-1);
	corners[2]->set( 1, 1,-1);
	corners[3]->set(-1, 1,-1);
	corners[4]->set(-1,-1, 1);
	corners[5]->set( 1,-1, 1);
	corners[6]->set( 1, 1, 1);
	corners[7]->set(-1, 1, 1);

	dragDir = new double[24];
	dragCorn = new double[96];

	topCorners = new Vector *[8];
	for(tmp=0;tmp<8;tmp++) topCorners[tmp]=new Vector();
	botCorners = new Vector *[8];
	for(tmp=0;tmp<8;tmp++) botCorners[tmp]=new Vector();
	
	temp.set(0,0,0);
	temp2.set(0,0,0);

	newCoord = new double [16];

	sideCols=new int [54];
	sideColsUndo=new int [54];

	// create a fresh cube
	newCube();
}

void Cube::dumpData()
{
	// used for debugging
}

int Cube::getTime()
{
	if ( timestarted )
	{
		return time->elapsed();
	}
	else	return 0;
}

int Cube::scattered()
{
	int ret=0;

	for(int tmp=0;tmp<54;tmp++)
	{
		if ( sideCols[tmp]!=(int)(tmp/9) ) ret++;
	}

	return ret;
}

void Cube::colorTwist(int sideNum, int quads, bool redraw )
{
	int i;
	int j;

	int l=0;
	int k = quads*2;

	// first move colors on twisted side...
	for(i=0;i<8;i++)
	{
		buffer[k]=sideCols[sideNum*9+circleOrder[i] ];
		k=(k+1)%8;
	}

	for(i=0;i<8;i++)
	{
		sideCols[sideNum*9+circleOrder[i] ]=buffer[i];
	}

	// then move colors on neighbour sides...

	k=quads*3;
	for(i=0;i<4;i++)
	{

	   for(j=0;j<4;j++)
	   {
		if (nextSide[nextSide[sideNum*4+i]*4+j]==sideNum) l=j;
	   }

	   for(j=0;j<3;j++)
	   {
		switch(l)
		{
		   case 0:	buffer[k] = 
				    sideCols[nextSide[sideNum*4+i]*9+j];
				break;
		   case 1:	buffer[k] = 
				    sideCols[nextSide[sideNum*4+i]*9+2+3*j];
				break;
		   case 2:	buffer[k] = 
				    sideCols[nextSide[sideNum*4+i]*9+8-j];
				break;
		   case 3:	buffer[k] = 
				    sideCols[nextSide[sideNum*4+i]*9+6-3*j];
				break;
		   default:	break;
		}
		k=(k+1)%12;
   	   }

	}
	
	k=0;
	for(i=0;i<4;i++)
	{
		for(j=0;j<4;j++)
		{
			if(nextSide[nextSide[sideNum*4+i]*4+j]==sideNum)
					l=j;
		}
		for (j=0;j<3;j++)
		{
		switch(l)
		{
			   case 0:	sideCols[nextSide[sideNum*4+i]*9+j]=buffer[k];
					break;
			   case 1:	sideCols[nextSide[sideNum*4+i]*9+2+3*j]=buffer[k];
					break;
			   case 2:	sideCols[nextSide[sideNum*4+i]*9+8-j]=buffer[k];
					break;
			   case 3:	sideCols[nextSide[sideNum*4+i]*9+6-3*j]=buffer[k];
					break;
			   default:	break;
		}
			k++;
		}
	// end for-loop
	}

	moves++;
	
	// time-statistics
	if ( !timestarted )
	{
		time->start();
		timestarted=true;
	}

	if(redraw) repaint(false);	

// end function
}

void Cube::fixBlock(Vector beye, Vector beX, Vector beY,
			Vector **bcorners, const int *bblocks, const int mode)
{
	int tmp;
	int i;
	int j;
	int k;

	light=beye;
	light*=-3;
	light+=beX;
	light-=beY;

	double center=grootte/2;
	double out=center*0.55;
	if ( out>50 ) out-=(out-50)*0.6;

	// note : here we could do some resizing...
	for(tmp=0;tmp<8;tmp++)	// project 3D coordinates onto 2D screen
	{
		newCoord[tmp*2]=(center+out*beX.scalProd( *bcorners[tmp] ) );
		newCoord[tmp*2+1]=(center-out*beY.scalProd( *bcorners[tmp] ) );
	}

	// go off all 6 sides....

	for(i=0;i<6;i++)
	{
		double sx=0,sy=0,sdxh=0,sdyh=0,sdxv=0,sdyv=0;
		
		// face toward us? then draw it!
		if ( sideVec[i]->scalProd(beye)>0.001 )
		{
			// used for shading
			if (shading)
			{
				k=(int)(9.6*(1-sideVec[i]->cosAngle(light) ) );
			}
			else k=19;

			// find corner coordinates
			for(j=0;j<4;j++)
			{
				rect.setPoint(j, (int)newCoord[2*sides[i*4+j]],
					(int)newCoord[2*sides[i*4+j]+1] );
			}
			
			pt->begin(map);
			pt->setPen( *edge );
			pt->setBrush ( *edge );
			pt->drawPolygon( rect );
			pt->end();

			// now draw colored fields
			sideW=bblocks[i*4+1]-bblocks[i*4];
			sideH=bblocks[i*4+3]-bblocks[i*4+2];

			if (sideW>0)
			{
				sx=newCoord[2*sides[i*4]];
				sy=newCoord[2*sides[i*4]+1];
				sdxh=(newCoord[2*sides[i*4+1]]-sx)/sideW;
				sdxv=(newCoord[2*sides[i*4+3]]-sx)/sideH;
				sdyh=(newCoord[2*sides[i*4+1]+1]-sy)/sideW;
				sdyv=(newCoord[2*sides[i*4+3]+1]-sy)/sideH;
				p=bblocks[i*4+2];
				for (n=0;n<sideH;n++)
				{
					q=bblocks[i*4];
					for(o=0;o<sideW;o++)
					{
						rect.setPoint(0,	(int)(sx+(o+0.1)*sdxh+(n+0.1)*sdxv),
									(int)(sy+(o+0.1)*sdyh+(n+0.1)*sdyv) );
						rect.setPoint(1,	(int)(sx+(o+0.9)*sdxh+(n+0.1)*sdxv),
									(int)(sy+(o+0.9)*sdyh+(n+0.1)*sdyv) );
						rect.setPoint(2,	(int)(sx+(o+0.9)*sdxh+(n+0.9)*sdxv),
									(int)(sy+(o+0.9)*sdyh+(n+0.9)*sdyv) );
						rect.setPoint(3,	(int)(sx+(o+0.1)*sdxh+(n+0.9)*sdxv),
									(int)(sy+(o+0.1)*sdyh+(n+0.9)*sdyv) );
						pt->begin(map);
						pt->setBrush(*colorList[20*sideCols[i*9+p*3+q]+k]);
						pt->setPen( *colorList[20*sideCols[i*9+p*3+q]+k] );
						pt->drawPolygon( rect );
						pt->end();
						q++;

					}
					p++;
				// end for-loop
				}
			// end if
			}

			// now we determine drag regions and directions
			double d=0,t1=0,t2=0,t3=0,t4=0,t5=0,t6=0;

			switch (mode)
			{

				case 0: // Just the normal cube
					t1=sx;
					t2=sy;
					t3=sdxh;
					t4=sdyh;
					t5=sdxv;
					t6=sdyv;
					for (j=0;j<4;j++)
					{
						dragCorn[8*dragReg]=t1;
						dragCorn[8*dragReg+4]=t2;
						dragCorn[8*dragReg+3]=t1+t5;
						dragCorn[8*dragReg+7]=t2+t6;
				
						t1=t1+t3*3;
						t2=t2+t4*3;	

						dragCorn[8*dragReg+1]=t1;
						dragCorn[8*dragReg+5]=t2;
						dragCorn[8*dragReg+2]=t1+t5;
						dragCorn[8*dragReg+6]=t2+t6;
						dragDir[dragReg*2]=t3*twistDir[i*4+j];
						dragDir[dragReg*2+1]=t4*twistDir[i*4+j];

						d=t3;
						t3=t5;
						t5=-d;
						d=t4;
						t4=t6;
						t6=-d;
						nearSide[dragReg]=nextSide[i*4+j];

						dragReg++;
					}
					break;


				case 1: // The large sub-cube
					break;

				case 2: // The small sub-cube (twistable part)

					if ((i!=twistSide)&&(sideW>0))
					{

						if (sideW==3) // Determine positive drag direction
						{
							if (bblocks[i*4+2]==0)
							{
								dragDir[dragReg*2]=sdxh*twistDir[i*4];
								dragDir[dragReg*2+1]=sdyh*twistDir[i*4];
							}
							else
							{
								dragDir[dragReg*2]=-sdxh*twistDir[i*4+2];
								dragDir[dragReg*2+1]=-sdyh*twistDir[i*4+2];
							}
						}
						else	
						{
							if (bblocks[i*4]==0)
							{
								dragDir[dragReg*2]=-sdxv*twistDir[i*4+3];
								dragDir[dragReg*2+1]=-sdyv*twistDir[i*4+3];
							}
							else
							{
								dragDir[dragReg*2]=sdxv*twistDir[i*4+1];
								dragDir[dragReg*2+1]=sdyv*twistDir[i*4+1];
							}
						}

						for (j=0;j<4;j++)
						{
							dragCorn[dragReg*8+j]=newCoord[2*sides[i*4+j]];
							dragCorn[dragReg*8+4+j]=newCoord[2*sides[i*4+j]+1];
						}
						nearSide[dragReg]=twistSide;

					dragReg++;
					}
					break;

				default:	break;
			// end switch-statement
			}


		// end if-construction 'if(sideVec.....'
		}		

	// end for-loop
	}

// end function fixBlock()
}

void Cube::cutUpCube()
{
	int i, j, k;

	bool check;
	
	for(i=0;i<8;i++)
	{
		topCorners[i]->copyVec( corners[i] );
		botCorners[i]->copyVec( corners[i] );
	}

	temp=sideVec[twistSide];
	temp2=temp;

	temp*=1.333333;
	temp2*=0.666666;
	
	for(i=0;i<8;i++)
	{
		check=false;
		for(j=0;j<4;j++)
			if (i==sides[twistSide*4+j] ) check=true;   // corner in twistside
		if (check)
			botCorners[i]->subVec( temp2 );
		else	topCorners[i]->addVec( temp );
	}

	// the sub-cubes need information about which fields belong to them	
	for(i=0;i<24;i++)
	{
		topBlocks[i]=mainBlocks[i];
		botBlocks[i]=mainBlocks[i];
	}

	for(i=0;i<6;i++)
	{
	    // large cube is blank on top
	    if( i ==twistSide )
	    {
		botBlocks[i*4+1]=0;
		botBlocks[i*4+3]=0;
	    }
	    else
	    {
		k=-1;
		for(j=0;j<4;j++)
			if ( nextSide[i*4+j]==twistSide) k=j;
		switch(k)
		{
			case 0:	// Up side?
				topBlocks[i*4+3]=1;
				botBlocks[i*4+2]=1;
				break;
			
			case 1:	// Right side?
				topBlocks[i*4]=2;
				botBlocks[i*4+1]=2;
				break;
			
			case 2:	// Down side?
				topBlocks[i*4+2]=2;
				botBlocks[i*4+3]=2;
				break;
			
			case 3:	// Left side?
				topBlocks[i*4+1]=1;
				botBlocks[i*4]=1;
				break;
			
			case -1: // None
				// small sub-cube in blank on
				topBlocks[i*4+1]=0;
				topBlocks[i*4+3]=0;
				break;
			default:	break;

		// end of switch
		}
	    // end of if-construction
	    }
	// end of for-loop (i)
	}

}


void Cube::mousePressEvent( QMouseEvent *mouse )
{
	lastX=mouse->x();
	lastY=mouse->y();
	OKtoDrag=true;
}

void Cube::mouseReleaseEvent( QMouseEvent * )
{
	int quads;
	double qu;
	// handle all the stuff....

	if (twisting)
	{
		twisting=false;
		phibase+=phi;
		phi=0;
		qu=phibase;
		while(qu<0) qu+=125.662;
		quads=( (int)(qu*3.183) );
		// close enough to a corner?
		if ( (  (quads%5)==0 ) || ( (quads%5)==4 ) )
		{
			quads=( (quads+1)/5)%4;
			if ( colDir[twistSide]<0 )
				quads=(4-quads) %4;
			phibase=0;
			// return cube the natural state
			naturalstate=true;
	
			// and shift corners
			colorTwist(twistSide, quads);
			stateChange();
		}
		repaint(false);
	}
}

void Cube::mouseMoveEvent( QMouseEvent *mouse )
{
	int i;
	double d;

	int x=mouse->x();
	int y=mouse->y();

	bool check;
	double x1,x2,y1,y2,alpha,beta;

	if( !twisting && (OKtoDrag) )
	{
		OKtoDrag=false;
		check=false;

		// check if we're in a drag region
		for (i=0;i<dragReg;i++)
		{
			x1=dragCorn[i*8+1]-dragCorn[i*8];
			x2=dragCorn[i*8+5]-dragCorn[i*8+4];
			y1=dragCorn[i*8+3]-dragCorn[i*8];
			y2=dragCorn[i*8+7]-dragCorn[i*8+4];
			alpha=(y2*(lastX-dragCorn[i*8])-y1*(lastY-dragCorn[i*8+4]))/
				(x1*y2-y1*x2);
			beta=(-x2*(lastX-dragCorn[i*8])+x1*(lastY-dragCorn[i*8+4]))/
				(x1*y2-y1*x2);
		
			if ((alpha>0)&&(alpha<1)&&(beta>0)&&(beta<1)) // We're in
			{
				currDragDir[0]=dragDir[i*2];
				currDragDir[1]=dragDir[i*2+1];
				d=currDragDir[0]*(double)(x-lastX)+
					currDragDir[1]*(double)(y-lastY);
				d=d*d/((currDragDir[0]*currDragDir[0]+currDragDir[1]*currDragDir[1])*
					(double)((x-lastX)*(x-lastX)+(y-lastY)*(y-lastY)));
				if (d>0.6)
				{
					check=true;
					twistSide=nearSide[i];
					i=100;
				}
			}

		}
	
		if (check) // We're twisting
		{
			if (naturalstate) // The cube still hasn't been split up
			{
				cutUpCube();
				naturalstate=false;
			}

			twisting=true;
			phi=0.02*(currDragDir[0]*(x-lastX)+currDragDir[1]*(y-lastY))/
				sqrt(currDragDir[0]*currDragDir[0]+currDragDir[1]*currDragDir[1]);
			repaint(false);

		}
	}
	
	OKtoDrag=false;
	if (!twisting) // normal rotation
	{
		dx=lastX-x; // Vertical shift
		temp=eX;
		temp.scalMult(  ((double)dx)*0.016   );
		eye+=temp;

		eX.vectorProd(eY,eye);
		eX.Normalize();
		eye.Normalize();

		dy=y-lastY; // Horizontal shift
		temp=eY;

		temp.scalMult( ((double)dy)*0.016 );
		eye+=temp;
		eY.vectorProd(eye,eX);
		eY.Normalize();
		eye.Normalize();
		lastX=x;
		lastY=y;
		repaint(false);
	}
	else // Twist, compute twisting angle phi
	{
	
		phi=0.02*(currDragDir[0]*(double)(x-lastX)+currDragDir[1]*(double)(y-lastY))/
			sqrt(currDragDir[0]*currDragDir[0]+currDragDir[1]*currDragDir[1]);
		repaint(false);
	}

}


void Cube::save()
{
	QString tmp;

	QString name = QFileDialog::getSaveFileName(0,"*.cube",this);

	if ( !name.isNull() )
	{
		QFile file ( name.data() );

		if ( file.open ( IO_Truncate | IO_WriteOnly ) )
		{
			tmp.sprintf("%s %s %s\n", "# Save-file for",
					 NAME, VERSION );
			file.writeBlock(tmp.data(), tmp.length() );
			tmp.sprintf("# Moves\n");
			file.writeBlock(tmp.data(), tmp.length() );
			tmp.sprintf("%d\n", moves);
			file.writeBlock(tmp.data(), tmp.length() );
			tmp.sprintf("# Time\n");
			file.writeBlock(tmp.data(), tmp.length() );
			if (timestarted)	tmp.sprintf("%d\n", time->elapsed() );
			else		tmp.sprintf("%d\n", 0 );
			file.writeBlock(tmp.data(), tmp.length() );
		
			// save the cube-data
			tmp.sprintf("# Cube data\n");
			file.writeBlock(tmp.data(), tmp.length() );
				for(int i=0;i<54;i++)
			{
				tmp.sprintf("%d\n", sideCols[i]);
				file.writeBlock(tmp.data(), tmp.length() );
			}
				file.close();
		}
		else
		{
			KMsgBox::message(this, CAPTION,
				klocale->translate("Could not open file.\n"
					"Cube has not been saved.") );
		}
	}
}

void Cube::load()
{
	QFile file;
	QString tmp;

	QString name = QFileDialog::getOpenFileName(0,"*.cube",this);
	if ( !name.isNull() )
	{

		file.setName ( name.data() );
	
		if ( file.open ( IO_ReadOnly ) )
		{
			QTextStream t ( &file );
		
			tmp = t.readLine();
			if ( !tmp.contains(NAME) )
			{
				KMsgBox::message(this, CAPTION,
					klocale->translate("Sorry, file is no"
					" Rubik's Kube save file.") );
			}
			else
			{
				// get moves
				tmp = t.readLine();
				tmp = t.readLine();
				moves = tmp.toInt();
			
				// get time
				tmp = t.readLine();
				tmp = t.readLine();
				QTime p;
				p.start();
				*time = p.addMSecs( tmp.toInt() );
		
				// Get the cube data
				tmp = t.readLine();

				for(int i=0;i<54;i++)
				{
				tmp=t.readLine();
				sideCols[i]=tmp.toInt();
				}
				file.close();

				naturalstate=true;
				emit stateChange();
			}
		}
		else
		{
			KMsgBox::message(this, CAPTION,
			klocale->translate("Could not open file.") );
		}
	}
}

void Cube::setMoves(int n) { moves=n ; }
int Cube::getMoves() { return moves ; }


